<?php

namespace Drupal\ssf_comment\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SsfConfigForm extends ConfigFormBase {
  
  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;
  
  
  public function __construct(ConfigFactoryInterface $config_factory, ModuleHandlerInterface $module_handler) {
    parent::__construct($config_factory);
    
    $this->moduleHandler = $module_handler;
  }
  
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_handler')
    );
  }
  
  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ssf_comment.settings'];
  }
  
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ssf_comment_settings_form';
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ssf_comment.settings');
    
    // Fieldset for check
    $form['check'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Thresholds'),
      '#description' => $this->t('Set thresholds that will be used to designate a comment to be spam or ham.<br>Values closer to 0 will more likely be ham, while values closer to 100 will more likely be spam.<br>The values in the gap between the thresholds will be designated as unknown.<br>(Internally values between 0 and 1 are used).'),
      '#collapsible' => FALSE,
    ];
    $form['check']['ssf_comment_ham_threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Ham threshold'),
      '#description' => $this->t('Classification of comments with a value below this threshold will be designated ham.'),
      '#default_value' => $config->get('ssf_comment_ham_threshold'),
      '#min' => 1,
      '#max' => 99,
      '#step' => 1,
    ];
    $form['check']['ssf_comment_spam_threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Spam threshold'),
      '#description' => $this->t('Classification of comments with a value above this threshold will be designated spam.'),
      '#default_value' => $config->get('ssf_comment_spam_threshold'),
      '#min' => 1,
      '#max' => 99,
      '#step' => 1,
    ];

    return parent::buildForm($form, $form_state);
    
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ssf_comment.settings');
    $config
      ->set('ssf_comment_ham_threshold', $form_state->getValue('ssf_comment_ham_threshold'))
      ->set('ssf_comment_spam_threshold', $form_state->getValue('ssf_comment_spam_threshold'))
      ->save();
    
    parent::submitForm($form, $form_state);
  }
  
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    
    if ($form_state->getValue('ssf_comment_ham_threshold') > $form_state->getValue('ssf_comment_spam_threshold')) {
      $message = $this->t('The threshold for ham cannot be higher than the threshold for spam.');
      $form_state->setError($form['ssf_comment_ham_threshold'], $message);
      $message = $this->t('The threshold for spam cannot be lower than the threshold for ham.');
      $form_state->setError($form['ssf_comment_spam_threshold'], $message);
    }
  }
  
}
